/*! BZkit 2.22.0 | http://www.getbzkit.com | (c) 2014 YOOtheme | MIT License */
(function(addon) {
    var component;

    if (window.BZkit) {
        component = addon(BZkit);
    }

    if (typeof define == "function" && define.amd) {
        define("bzkit-accordion", ["bzkit"], function(){
            return component || addon(BZkit);
        });
    }
})(function(BZ){

    "use strict";

    BZ.component('accordion', {

        defaults: {
            showfirst  : true,
            collapse   : true,
            animate    : true,
            easing     : 'swing',
            duration   : 300,
            toggle     : '.bz-accordion-title',
            containers : '.bz-accordion-content',
            clsactive  : 'bz-active'
        },

        boot:  function() {

            // init code
            BZ.ready(function(context) {

                setTimeout(function(){

                    BZ.$("[data-bz-accordion]", context).each(function(){

                        var ele = BZ.$(this);

                        if(!ele.data("accordion")) {
                            BZ.accordion(ele, BZ.Utils.options(ele.attr('data-bz-accordion')));
                        }
                    });

                }, 0);
            });
        },

        init: function() {

            var $this = this;

            this.element.on('click.bzkit.accordion', this.options.toggle, function(e) {

                e.preventDefault();

                $this.toggleItem(BZ.$(this).data('wrapper'), $this.options.animate, $this.options.collapse);
            });

            this.update();

            if (this.options.showfirst) {
                this.toggleItem(this.toggle.eq(0).data('wrapper'), false, false);
            }
        },

        toggleItem: function(wrapper, animated, collapse) {

            var $this = this;

            wrapper.data('toggle').toggleClass(this.options.clsactive);

            var active = wrapper.data('toggle').hasClass(this.options.clsactive);

            if (collapse) {
                this.toggle.not(wrapper.data('toggle')).removeClass(this.options.clsactive);
                this.content.not(wrapper.data('content')).parent().stop().css('overflow', 'hidden').animate({ height: 0 }, {easing: this.options.easing, duration: animated ? this.options.duration : 0}).attr('aria-expanded', 'false');
            }

            wrapper.stop().css('overflow', 'hidden');

            if (animated) {

                wrapper.animate({ height: active ? getHeight(wrapper.data('content')) : 0 }, {easing: this.options.easing, duration: this.options.duration, complete: function() {

                    if (active) {
                        wrapper.css({'overflow': '', 'height': 'auto'});
                        BZ.Utils.checkDisplay(wrapper.data('content'));
                    }

                    $this.trigger('display.bz.check');
                }});

            } else {

                wrapper.height(active ? 'auto' : 0);

                if (active) {
                    wrapper.css({'overflow': ''});
                    BZ.Utils.checkDisplay(wrapper.data('content'));
                }

                this.trigger('display.bz.check');
            }

            // Update ARIA
            wrapper.attr('aria-expanded', active);

            this.element.trigger('toggle.bz.accordion', [active, wrapper.data('toggle'), wrapper.data('content')]);
        },

        update: function() {

            var $this = this, $content, $wrapper, $toggle;

            this.toggle = this.find(this.options.toggle);
            this.content = this.find(this.options.containers);

            this.content.each(function(index) {

                $content = BZ.$(this);

                if ($content.parent().data('wrapper')) {
                    $wrapper = $content.parent();
                } else {
                    $wrapper = BZ.$(this).wrap('<div data-wrapper="true" style="overflow:hidden;height:0;position:relative;"></div>').parent();

                    // Init ARIA
                    $wrapper.attr('aria-expanded', 'false');
                }

                $toggle = $this.toggle.eq(index);

                $wrapper.data('toggle', $toggle);
                $wrapper.data('content', $content);
                $toggle.data('wrapper', $wrapper);
                $content.data('wrapper', $wrapper);
            });

            this.element.trigger('update.bz.accordion', [this]);
        }

    });

    // helper

    function getHeight(ele) {

        var $ele = BZ.$(ele), height = "auto";

        if ($ele.is(":visible")) {
            height = $ele.outerHeight();
        } else {

            var tmp = {
                position   : $ele.css("position"),
                visibility : $ele.css("visibility"),
                display    : $ele.css("display")
            };

            height = $ele.css({position: 'absolute', visibility: 'hidden', display: 'block'}).outerHeight();

            $ele.css(tmp); // reset element
        }

        return height;
    }

    return BZ.accordion;
});
