/*! BZkit 2.22.0 | http://www.getbzkit.com | (c) 2014 YOOtheme | MIT License */
/*
 * Based on Nestable jQuery Plugin - Copyright (c) 2012 David Bushell - http://dbushell.com/
 */
(function(addon) {

    var component;

    if (window.BZkit) {
        component = addon(BZkit);
    }

    if (typeof define == "function" && define.amd) {
        define("bzkit-calendar", ["bzkit"], function(){
            return component || addon(BZkit);
        });
    }

})(function(BZ) {

    "use strict";
	
	var $prices 	= [],
		$win    	= BZ.$win,
		$container;

    BZ.component('calendar', {

        defaults: {
            dateformat: 'Y-m-d',
			dateshort: 'm-d',
			baselink : '',
			oid: '',
			show: 2,
			active: '',
			showprices: false,
			action: true,
			calculation: false,
			pricelist: false,
			alternatives: false,
			alternativeactive: false,
			calcpriceformat: 'always',
			priceformat: 'always',
			priceincl: false,
			type: 'none',
			highlight: 0,
			highlightstart: 0
        },
		
		arrival: '',
		departure: '',
		active: 1,
		max: 1,
		nights: 0,

        boot: function() {

            // init code
            BZ.ready(function(context) {

                BZ.$("[data-bz-calendar]", context).each(function(){

                    var ele = BZ.$(this);

                    if (!ele.data("calendar")) {
                        BZ.calendar(ele, BZ.Utils.options(ele.attr("data-bz-calendar")));
                    }
                });
            });
        },

        init: function() {

            var $this = this,
				r	  = [];
			
			$prices = this.options.prices ? this.options.prices : $win.bzcalendarprices;
			
			// get active arrival and departure
			var a 			= BZ.Utils.options(this.element.data('period'));
			this.arrival 	= a.arrival;
			this.departure 	= a.departure;
			
			// get active || shouldn't we do this by settings from calendar class
			this.active = this.find('[data-month]:not(.bz-hidden)').first().data('index') || this.active;
			this.max 	= this.find('[data-month]').last().data('index') || this.max;
			this.max 	= this.max - (this.options.show - 1);
			this.max 	= this.max < 1 ? 1 : this.max;
			this.nights	= this.find('[data-type]').val() || 0;
			
			this.element.removeAttr('data-bz-calendar data-period');
			
			if ( this.options.alternatives ) {
				this.alternativerow = this.find('[data-alternatives] > div:first-child');
			}
			
			if ( this.find('[data-tooltip-original]')) {
				this.tooltip = this.find('[data-tooltip-original]').eq(0).data('tooltip-original');
				this.find('[data-tooltip-original]').removeAttr('data-tooltip-original');
			}
			else {
				this.tooltip = false;
			}
			
			this.reset();
			this.activate(this.active);
			
			this.on('click', '[data-arrival]', function () {
				$this.onArrivalClick(BZ.$(this));
			});
			
			this.on('click', '[data-action]', function () {
				$this.onActionClick(BZ.$(this));
			});
			
			this.on('change', '[data-field="departure"] select', function () {
				$this.onDepartureChange(BZ.$(this));
			});
			
			this.on('change', '[data-persons]', function () {
				$this.onPersonChange(BZ.$(this));
			});
			
			this.on('change', '[data-type]', function () {
				$this.onTypeChange(BZ.$(this));
			});
			
			this.on('click', '[data-next]', function() {
				$this.activate($this.active + 1);
			});
			
			this.on('click', '[data-previous]', function() {
				$this.activate($this.active - 1);
			});
			
			this.on('click', '[data-calendar-reset]', function() {
				var a = $this.find('[data-stamp="'+$this.arrival+'"]');
				if ( a && a.hasClass('isactive') ) {
					a.trigger('click');
				}
			});
			
			if ( this.options.type == 'both' && this.options.highlightstart ) {
				this.find('[data-stamp="'+this.arrival+'"]').trigger('click');
			}
			
			if ( this.options.highlightstart ) {
				this.highlight(this.arrival, this.departure);
			} 
        },
		
		pushprices: function(n)
		{
			$prices  = this.processprices(n);
			
			var $this = this,
				o	  = this.options,
				s 	  = this.find('.calendar-price');
			
			// loop each price displayed and replace it
			BZ.$.each(s, function() {
				var e = BZ.$(this),
					a = e.data('from'),
					d = e.data('until'),
					v = $this.getprice(a,d);
				
				e.html($this.priceformat((o.priceincl ? v.total : v.nettotal), o.priceformat));
			});
			
			this.updatePricelist();
		},
		
		reset: function() {
			
			var $this = this,
				o 	  = this.options,
				r	  = [],
				c	  = this.find('[data-stamp]');
			
			// first remove all old data
			BZ.$.each(c, function() {
				var d = BZ.$(this);
				d.removeAttr('data-action data-arrival').removeClass('isactive highlight highlight-start highlight-end').html(d.data('day'));
			});
			
			// process price calculating 
			$prices  = this.processprices($prices);
						
			// loop each price
			BZ.$.each($prices, function(k,v) {
								
				// on type is none we're done
				if ( o.type == 'none' || (o.type == 'type' && v.nights != $this.nights) ) {
					return;
				}
				
				// add each arrival just once
				if ( !BZ.Utils.inArray(v.arrival, r) ) {
										
					r.push(v.arrival);

					// find the date
					var d = $this.find('[data-stamp="'+v.arrival+'"]');
				
					if ( d ) {
						
						// add a data attribute for calculation or departure search
						if ( ((o.calculation || o.alternatives) && o.action) || o.type == 'both' ) {
							if ( o.type == 'both' ) {
								BZ.Utils.data(d, 'arrival', '', true);
							}
							else {
								BZ.Utils.data(d, 'action', v.departure);
							}
						}
						
						// create a link element
						var a = BZ.$('<a>').addClass('arrival-date').html(d.data('day'));
						
						// add price only if asked and type is not both (arrival and departures)
						if ( o.showprices && o.type != 'both' ) {
							BZ.$('<span>').addClass('calendar-price')
										  .attr('data-from', v.arrival).attr('data-until', v.departure)
										  .html($this.priceformat((o.priceincl ? v.total : v.nettotal), o.priceformat)).appendTo(a);
						}
						
						// add link if needed
						if ( o.action && v.link && !(o.calculation || o.alternatives || o.type == 'both')) {
							a.attr('href', v.link);
							
							if ( o.highlight ) {
								a.on('mouseenter', function() {
									$this.highlight(v.arrival, v.departure);
								}).on('mouseleave', function() {
									$this.clearhighlights();
								});	
							}
						}
						
						// fill up the cell
						d.html(a);
					}
				}
			});
		},
		
		activate: function(i) {
			
			var i = i < 1 ? 1 : ( i > this.max ? this.max : i),
				l = i + (this.options.show - 1),
				m = this.find('[data-month]');
			
			BZ.$.each(m, function() {
				var m = BZ.$(this),
					c = parseFloat(m.data('index'));
					m.toggleClass('bz-hidden', !(c >= i && c <= l) );
			});
			
			this.active = i;
			
			this.find('[data-previous]').toggleClass('inactive', this.active == 1);
			this.find('[data-next]').toggleClass('inactive', this.active >= this.max );
		},
		
		onArrivalClick: function(a) {
						
			var $this 	= this,
				o 		= this.options,
				a 		= BZ.$(a),
				c 		= a.data('stamp'),
				h 		= a.hasClass('isactive'),
				r 		= [],
				m 		= c,
				p		= this.getprice(c),
				x 		= new Date( p.arrival * 1000).format('n'),
				y 		= 0;
				
			
			// if we deactivate an arrival reset
			this.reset();
			
			if ( !h ) {
				a.addClass('isactive');
				
				// loop prices finding departures
				BZ.$.each($prices, function(k,v) {

					if ( c == v.arrival && !BZ.Utils.inArray(v.departure, r) ) {

						r.push(v.departure);

						var d = $this.find('[data-stamp="'+v.departure+'"]');

						if ( d ) {
							
							// set new largest departure month
							var dy = new Date( v.departure * 1000).format('n');
							if ( dy > y ) {
								y = dy;
							}
							
							BZ.Utils.data(d, 'action', v.arrival);
														
							// create a link element
							var a = BZ.$('<a>').addClass('departure-date').html(d.data('day'));

							// add price only if asked and type is not both (arrival and departures)
							if ( o.showprices ) {
								BZ.$('<span>').addClass('calendar-price')
											  .attr('data-from', v.arrival).attr('data-until', v.departure)
											  .html($this.priceformat((o.priceincl ? v.total : v.nettotal), o.priceformat)).appendTo(a);
							}

							// add link if needed
							if ( v.link && !(o.calculation)) {
								a.attr('href', v.link);
								
								if ( o.highlight ) {
									a.on('mouseenter', function() {
										$this.highlight(v.arrival, v.departure);
									}).on('mouseleave', function() {
										$this.clearhighlights();
									});	
								}
							}

							// fill up the cell
							d.html(a);
							
							m = v.departure > m ? v.departure : m;
						}
					}
					
				});
				
				// update?
				if ( p ) {
					this.updates(p.arrival, p.departure);
				}
				
				// move calenar ?
				/*
				if ( y > x && this.options.show > 1) {
					this.find('[data-next]').trigger('click');
				}
				*/
				
				// remove arrivals overlapping departures
				this.removeArrivals(c,m);
			}
		},
		
		onActionClick: function(c) {
			
			var t = c.data('action'),
				d = c.data('stamp'),
				a = t > d ? d : t,
				d = d < t ? t : d;
			
			this.updates(a,d);
		},
		
		updates: function(a,d)
		{
			if (a) {
				this.updateAlternatives(a);
				this.changePricelist(a);
			}
			if ( a && d ) {
				this.updateCalc(a,d);		
				this.highlight(a,d);
			}
			
			this.hideempty();
		},
		
		onDepartureChange: function(c) {
			
			var a = this.arrival,
				d = BZ.$(c).val();
		
			this.updates(a, d);
		},
		
		onTypeChange: function(c) {
			
			this.nights = BZ.$(c).val();
			this.reset();
			
			var d = this.arrival + (this.nights * 86400),
				p = this.getprice(this.arrival, d);
			
			if (p) {
				this.departure = d;
				this.updates(this.arrival, this.departure);
			}
		},
		
		onPersonChange: function(c) {
			
			var $this = this,
				e = BZ.$(c),
				p = e.val();
			
			if ( !this.element.hasClass('fetching') ) {
				
				this.element.addClass('fetching');
		
				BZ.$.ajax({
					type: "POST",
					url: BOOKZO_URL + 'ajax.php',
					data: 'format=ajax&layout=widgets&widget=calendar&task=getprices&bookzolangid='+BOOKZO_LANGID + '&persons=' + p + '&oid=' + $this.options.oid,
					dataType: 'json',
					success: function(r){
						if ( r.success ) {
							var a = BZ.Utils.options(r.data);
							if ( a.link ) {
								$this.options.baselink = a.link;
							}
							if ( a.prices ) {
								$this.pushprices(a.prices);
								$this.updates($this.arrival, $this.departure);
							}
							$this.element.removeClass('fetching');
						}
						else {
							$this.element.removeClass('fetching');
						}
					},
					error: function(){
						$this.element.removeClass('fetching');
					}
				});
			}
			
		},
		
		updateAlternatives: function(a)
		{
			if ( this.options.alternatives ) {
				var $this = this,
					o = this.options,
					n = this.find('[data-alternatives]');

				n.html('');

				if ( a ) {	
					var t = this.getprices(a);

					if ( t.length ) {
						BZ.$.each(t, function(k,v) {
							var b = BZ.$( $this.alternativerow).clone();
								b.find('> div').removeClass('active bz-panel-box-primary');
							
							if ( o.alternativeactive && v.arrival == $this.arrival && v.departure == $this.departure ) {
								b.find('> div').addClass('active bz-panel-box-primary');
							}
							
							b.find('[data-field="duration"]').html( v.name );
							b.find('[data-field="arrival"]').html( v.arrivalshort );
							b.find('[data-field="departure"]').html( v.departureshort );
							b.find('[data-field="before"]').html( $this.priceformat( v.before, o.calcpriceformat) );
							b.find('[data-field="total"]').html( $this.priceformat( v.total, o.calcpriceformat) );
							b.find('[data-field="link"]').attr('href', v.link );
							
							$this.createtooltip(b, v);

							n.append(b);
						});
					}
				
				}
			}
		},
		
		updateCalc: function(a,d) {
			
			var $this = this,
				o = this.options,
				p = this.getprice(a,d);

			if ( o.calculation ) {
				var c = this.find('[data-calculation]'),
					d = c.find('[data-field="departure"]'),
					s = BZ.$('select', d);
				

				// update fields
				c.find('[data-field="price"]').html( $this.priceformat(p.price, o.calcpriceformat) ) ;
				c.find('[data-field="costs"]').html( $this.priceformat( p.costs, o.calcpriceformat) );
				c.find('[data-field="discount"]').html( $this.priceformat(p.discount, o.calcpriceformat) );
				c.find('[data-field="before"]').html( $this.priceformat(p.before, o.calcpriceformat) );
				c.find('[data-field="total"]').html( $this.priceformat( p.total, o.calcpriceformat) );
				c.find('[data-field="duration"]').html( p.name );
				c.find('[data-field="link"]').attr('href', p.link );
				c.find('[data-field="arrival"]').html( p.arrivalformat );

				this.createtooltip(c, p);
				
				// update departurefield
				if ( s.length ) {
					s.empty();				
					BZ.$.each($prices, function(k,v) {
						if ( v.arrival == p.arrival ) {
							var o = BZ.$('<option></option>').attr('value', v.departure).text(v.departureformat);
							if ( v.departure == p.departure ) {
								o.attr('selected', true);
							}
							s.append(o);
						}
					});
				}
				else {
					d.html( p.departureformat );
				}
			}

			// set new active fields
			$this.arrival = p.arrival;
			$this.departure = p.departure;
		},
		
		updatePricelist: function()
		{
			var $this = this,
				o 	  = this.options;
		
			if ( o.pricelist ) {
				var c 	  = this.find('[data-pricelist]');	
			
				BZ.$.each($prices, function(k,v) {
					var r = c.find('[data-pricelist-stamp="'+v.arrival+'-'+v.departure+'"]');

					r.find('[data-field="before"]').html( $this.priceformat((o.priceincl ? v.before : v.netbefore), o.priceformat) );
					r.find('[data-field="total"]').html( $this.priceformat((o.priceincl ? v.total : v.nettotal), o.priceformat) );
					r.find('[data-field="link"]').attr('href', v.link);
				});
			}
		},
		
		createtooltip: function(e, p)
		{
			var e = BZ.$(e),
				o = this.options,
				s = e.find('[data-bz-tooltip]');
			
			if ( this.tooltip && s) {
				
				var n = BZ.$(this.tooltip).clone();

				n.find('[data-field="price"]').html( this.priceformat(p.price, o.calcpriceformat) ) ;
				n.find('[data-field="costs"]').html( this.priceformat( p.costs, o.calcpriceformat) );
				n.find('[data-field="discount"]').html( this.priceformat(p.discount, o.calcpriceformat) );
				n.find('[data-field="total"]').html( this.priceformat( p.total, o.calcpriceformat) );

				BZ.Utils.data(s, 'tooltip-text', BZ.Utils.objectToString(this.hideempty(n)));
			}
		},
		
		highlight: function(a,b) {
			
			if ( this.options.highlight ) {
				var $this = this,
					t = this.find('[data-stamp]');

				this.clearhighlights();

				BZ.$.each(t, function() {
					var c = BZ.$(this),
						d = c.data('stamp');

					if ( d >= a && d <= b ) { c.addClass('highlight');}
					if ( d == a ) { c.addClass('highlight-start'); }
					if ( d == b ) { c.addClass('highlight-end'); }

				});
			}
		},
		
		clearhighlights: function(a,b) {
			this.find('[data-stamp]').removeClass('highlight highlight-start highlight-end');
		},
		
		hideempty: function(a) {
			var n = a ? a : this,
				f = n.find('[data-check]'),
				z = this.priceformat(0, this.options.calcpriceformat),
				y = this.priceformat(0, this.options.priceformat);
			
			BZ.$.each(f, function(k,v){
				var e = BZ.$(v),
					d = e.html(),
					p = e.closest('.parentfield'),
					s = !( !d.length || d == z || d == y);
				
				p.toggleClass('bz-hidden', !s);
			});
			
			if ( a ) return n;
		},
		
		removeArrivals: function(a,b)
		{
			var $this = this,
				t = this.find('[data-arrival]');
			
			BZ.$.each(t, function() {
				var c = BZ.$(this),
					d = c.data('stamp'),
					h = c.data('day');
				
				if ( d > a && d <= b ) {
					c.removeAttr('data-arrival').find('.arrival-date').replaceWith(h);
				}
			});
		},
		
		getprice: function(a,d)
		{
			var p = false;
			
			BZ.$.each($prices, function(k,v) {
				if ( (v.arrival == a && !d) || (v.arrival == a && v.departure == d) ) {
					p = v; return false;
				}
			});
			
			return p;
		},
		
		getprices: function(a)
		{
			var p = [];
			
			BZ.$.each($prices, function(k,v) {
				if ( v.arrival == a ) {
					p.push(v);
				}
			});
			
			return p;
		},
		
		processprices: function(p)
		{
			var $this = this;
			
			BZ.$.each(p, function(k,v) {
				p[k] = $this.processprice(v);
			});
			
			return p;
		},
		
		// get the price html for a cell
		processprice: function(p)
		{
			var a = new Date( p.arrival * 1000),
				d = new Date( p.departure * 1000 );
			
			p.price      		= BZ.Utils.float(p.price);
			p.discount   		= BZ.Utils.float(p.discount || 0);
			p.costs	     		= BZ.Utils.float(p.costs || 0);
			p.total	     		= p.price + p.costs - p.discount;
			p.nettotal 	 		= p.price - p.discount;
			p.before			= p.discount ? p.price + p.costs : 0;
			p.netbefore			= p.discount ? p.price : 0;	
			p.arrivalval 		= a.format('Y-m-d');
			p.departureval		= d.format('Y-m-d');
			p.arrivalformat 	= a.format(this.options.dateformat);
			p.departureformat 	= d.format(this.options.dateformat);
			p.arrivalshort 		= a.format(this.options.dateshort);
			p.departureshort 	= d.format(this.options.dateshort);
			p.link				= this.options.baselink ? this.options.baselink + '&arrival=' + p.arrivalval + '&departure=' + p.departureval : '';
			
			return p;
		},
		
		changePricelist: function(a)
		{
			if ( this.options.pricelist ) {		
				var m = new Date(a * 1000).format('Y-m-01');
				this.find('[data-pricelist]').find('[data-pricelist-month="'+m+'"]').trigger('click.bzkit.slideshow');
			}
		},
		
		priceformat: function(p,t) {
			
			var f = BZ.Utils.cast('float', p),
				r = BZ.Utils.cast('int', p),
				d = f.toFixed(2).split('.')[1];
			
			switch (t) {
				default:
				case 'always': return BZ.Utils.thousands(r, '.') + ',' + d;	
				case 'round': return BZ.Utils.thousands(Math.round(f), '.');
				case 'nozero':
				case 'zerochange':
					if ( f == r ) {
						return BZ.Utils.thousands(r, '.') + ( t == 'zerochange' ? ',-' : '' );
					}
					else {
						return BZ.Utils.thousands(r, '.') + ',' + d;
					}
			}	
		}

    });

    return BZ.calendar;
});

/*
 * date.js 
 */
(function() {
	
	Date.shortMonths = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
    Date.longMonths = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
    Date.shortDays = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];
    Date.longDays = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
	
    // defining patterns
    var replaceChars = {
        // Day
        d: function() { return (this.getDate() < 10 ? '0' : '') + this.getDate(); },
        D: function() { return Date.shortDays[this.getDay()]; },
        j: function() { return this.getDate(); },
        l: function() { return Date.longDays[this.getDay()]; },
        N: function() { return (this.getDay() == 0 ? 7 : this.getDay()); },
        S: function() { return (this.getDate() % 10 == 1 && this.getDate() != 11 ? 'st' : (this.getDate() % 10 == 2 && this.getDate() != 12 ? 'nd' : (this.getDate() % 10 == 3 && this.getDate() != 13 ? 'rd' : 'th'))); },
        w: function() { return this.getDay(); },
        z: function() { var d = new Date(this.getFullYear(),0,1); return Math.ceil((this - d) / 86400000); }, // Fixed now
        // Week
        W: function() {
            var target = new Date(this.valueOf());
            var dayNr = (this.getDay() + 6) % 7;
            target.setDate(target.getDate() - dayNr + 3);
            var firstThursday = target.valueOf();
            target.setMonth(0, 1);
            if (target.getDay() !== 4) {
                target.setMonth(0, 1 + ((4 - target.getDay()) + 7) % 7);
            }
            var retVal = 1 + Math.ceil((firstThursday - target) / 604800000);
          
            return (retVal < 10 ? '0' + retVal : retVal);
        },
        // Month
        F: function() { return Date.longMonths[this.getMonth()]; },
        m: function() { return (this.getMonth() < 9 ? '0' : '') + (this.getMonth() + 1); },
        M: function() { return Date.shortMonths[this.getMonth()]; },
        n: function() { return this.getMonth() + 1; },
        t: function() {
            var year = this.getFullYear(), nextMonth = this.getMonth() + 1;
            if (nextMonth === 12) {
                year = year++;
                nextMonth = 0;
            }
            return new Date(year, nextMonth, 0).getDate();
        },
        // Year
        L: function() { var year = this.getFullYear(); return (year % 400 == 0 || (year % 100 != 0 && year % 4 == 0)); },   // Fixed now
        o: function() { var d  = new Date(this.valueOf());  d.setDate(d.getDate() - ((this.getDay() + 6) % 7) + 3); return d.getFullYear();}, //Fixed now
        Y: function() { return this.getFullYear(); },
        y: function() { return ('' + this.getFullYear()).substr(2); },
        // Time
        a: function() { return this.getHours() < 12 ? 'am' : 'pm'; },
        A: function() { return this.getHours() < 12 ? 'AM' : 'PM'; },
        B: function() { return Math.floor((((this.getUTCHours() + 1) % 24) + this.getUTCMinutes() / 60 + this.getUTCSeconds() / 3600) * 1000 / 24); }, // Fixed now
        g: function() { return this.getHours() % 12 || 12; },
        G: function() { return this.getHours(); },
        h: function() { return ((this.getHours() % 12 || 12) < 10 ? '0' : '') + (this.getHours() % 12 || 12); },
        H: function() { return (this.getHours() < 10 ? '0' : '') + this.getHours(); },
        i: function() { return (this.getMinutes() < 10 ? '0' : '') + this.getMinutes(); },
        s: function() { return (this.getSeconds() < 10 ? '0' : '') + this.getSeconds(); },
        u: function() { var m = this.getMilliseconds(); return (m < 10 ? '00' : (m < 100 ?
    '0' : '')) + m; },
        // Timezone
        e: function() { return /\((.*)\)/.exec(new Date().toString())[1]; },
        I: function() {
            var DST = null;
                for (var i = 0; i < 12; ++i) {
                        var d = new Date(this.getFullYear(), i, 1);
                        var offset = d.getTimezoneOffset();

                        if (DST === null) DST = offset;
                        else if (offset < DST) { DST = offset; break; }                     
						else if (offset > DST) break;
                }
                return (this.getTimezoneOffset() == DST) | 0;
            },
        O: function() { return (-this.getTimezoneOffset() < 0 ? '-' : '+') + (Math.abs(this.getTimezoneOffset() / 60) < 10 ? '0' : '') + Math.floor(Math.abs(this.getTimezoneOffset() / 60)) + (Math.abs(this.getTimezoneOffset() % 60) == 0 ? '00' : ((Math.abs(this.getTimezoneOffset() % 60) < 10 ? '0' : '')) + (Math.abs(this.getTimezoneOffset() % 60))); },
        P: function() { return (-this.getTimezoneOffset() < 0 ? '-' : '+') + (Math.abs(this.getTimezoneOffset() / 60) < 10 ? '0' : '') + Math.floor(Math.abs(this.getTimezoneOffset() / 60)) + ':' + (Math.abs(this.getTimezoneOffset() % 60) == 0 ? '00' : ((Math.abs(this.getTimezoneOffset() % 60) < 10 ? '0' : '')) + (Math.abs(this.getTimezoneOffset() % 60))); }, // Fixed now
        T: function() { return this.toTimeString().replace(/^.+ \(?([^\)]+)\)?$/, '$1'); },
        Z: function() { return -this.getTimezoneOffset() * 60; },
        // Full Date/Time
        c: function() { return this.format("Y-m-d\\TH:i:sP"); }, // Fixed now
        r: function() { return this.toString(); },
        U: function() { return this.getTime() / 1000; }
    };

    // Simulates PHP's date function
    Date.prototype.format = function(format) {
        var date = this;
        return format.replace(/(\\?)(.)/g, function(_, esc, chr) {
            return (esc === '' && replaceChars[chr]) ? replaceChars[chr].call(date) : chr;
        });
    };

}).call(this);