<?php
/**
 * @package     Joomla.Libraries
 * @subpackage  HTML
 *
 * @copyright   Copyright (C) 2005 - 2014 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('_BOOKZO') or die;

/**
 * Utility class for JavaScript behaviors
 *
 * @package     Joomla.Libraries
 * @subpackage  HTML
 * @since       1.5
 */
abstract class BHtmlBehavior
{
	/**
	 * Array containing information for loaded files
	 *
	 * @var    array
	 * @since  2.5
	 */
	protected static $loaded = array();


	/**
	 * Add unobtrusive JavaScript support for a showon control.
	 *
	 * @return  void
	 * @since   1.0
	 */
	public static function tooltip()
	{
		// Only load once
		if (isset(static::$loaded[__METHOD__]))
		{
			return;
		}

		// Include jQuery
		$bookzo = Bookzo::getInstance();
		
		$bookzo->adapter->jQuery(true);
		$bookzo->adapter->addScript('assets:jui/js/tooltip.js');
		$bookzo->adapter->addStylesheet('assets:jui/css/tooltip.css');

		static::$loaded[__METHOD__] = true;
	}
	
	/**
	 * Add unobtrusive JavaScript support for a showon control.
	 *
	 * @return  void
	 * @since   1.2.15
	 */
	public static function showon()
	{
		// Only load once
		if (isset(static::$loaded[__METHOD__]))
		{
			return;
		}

		// Include jQuery
		$bookzo = Bookzo::getInstance();
		$bookzo->adapter->jQuery(true);
		$bookzo->adapter->addScript('assets:jui/js/showon.js');

		static::$loaded[__METHOD__] = true;
	}

	/**
	 * Add unobtrusive JavaScript support for a calendar control.
	 *
	 * @return  void
	 * @since   1.0
	 */
	public static function datepicker()
	{
		// Only load once
		if (isset(static::$loaded[__METHOD__]))
		{
			return;
		}

		// Include jQuery
		$bookzo = Bookzo::getInstance();
		
		$bookzo->adapter->jQuery(true);
		$bookzo->adapter->addScript('assets:jui/js/datepicker.js');
		$bookzo->adapter->addStylesheet('assets:jui/css/datepicker.css');

		static::$loaded[__METHOD__] = true;
	}
	
	/**
	 * Add unobtrusive JavaScript support for a calendar control.
	 *
	 * @return  void
	 * @since   1.0
	 */
	public static function calendar()
	{
		// Only load once
		if (isset(static::$loaded[__METHOD__]))
		{
			return;
		}

		// Include jQuery
		$bookzo = Bookzo::getInstance();
		
		$bookzo->adapter->jQuery(true);
		$bookzo->adapter->addScript('assets:jui/js/calendar.js');
		
		// add custom jsdates in correct language
		$js		 = array();
		$js[]	 = "Date.shortMonths = ['".implode("','", $bookzo->date->getMonthList('short'))."']";
		$js[]	 = "Date.longMonths = ['".implode("','", $bookzo->date->getMonthList())."']";
		$js[]	 = "Date.shortDays = ['".implode("','", $bookzo->date->getDayList('short'))."']";
		$js[]	 = "Date.longDays = ['".implode("','", $bookzo->date->getDayList())."']";
		
		$bookzo->adapter->addScriptDeclaration( implode("\n", $js));

		static::$loaded[__METHOD__] = true;
	}
	
	/**
	 * Add unobtrusive JavaScript support for a calendar control.
	 *
	 * @return  void
	 * @since   1.1.0
	 */
	public static function nestable()
	{
		// Only load once
		if (isset(static::$loaded[__METHOD__]))
		{
			return;
		}

		// Include jQuery
		$bookzo = Bookzo::getInstance();
		
		$bookzo->adapter->jQuery(true);
		$bookzo->adapter->addScript('assets:jui/js/nestable.js');
		$bookzo->adapter->addStylesheet('assets:jui/css/nestable.css');

		static::$loaded[__METHOD__] = true;
	}
	
	/**
	 * Add unobtrusive JavaScript support for a calendar control.
	 *
	 * @return  void
	 * @since   1.1.0
	 */
	public static function chosen()
	{
		// Only load once
		if (isset(static::$loaded[__METHOD__]))
		{
			return;
		}

		// Include jQuery
		$bookzo = Bookzo::getInstance();
		
		$bookzo->adapter->jQuery(true);
		$bookzo->adapter->addScript('assets:jui/js/chosen.js');
		$bookzo->adapter->addStylesheet('assets:jui/css/chosen.css');

		static::$loaded[__METHOD__] = true;
	}


	/**
	 * Add unobtrusive JavaScript support for a color picker.
	 *
	 * @return  void
	 * @since   1.0
	 */
	public static function colorpicker()
	{
		// Only load once
		if (isset(static::$loaded[__METHOD__]))
		{
			return;
		}

		// Include jQuery
		$bookzo = Bookzo::getInstance();
		$bookzo->adapter->jQuery(true);

		// Include ui elements
		$bookzo->adapter->addScript('assets:jui/js/minicolors.min.js');
		$bookzo->adapter->addStyleSheet('assets:jui/css/minicolors.css');

		$bookzo->adapter->addScriptDeclaration("
				jQuery(document).ready(function (){
					jQuery('.minicolors').each(function() {
						jQuery(this).minicolors({
							control: jQuery(this).attr('data-control') || 'hue',
							position: jQuery(this).attr('data-position') || 'right',
							theme: 'bootstrap'
						});
					});
				});
			"
		);

		static::$loaded[__METHOD__] = true;
	}
}
