<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

/**
 * Bz data class.
 *
 * This class acts as parent for all data classes
 * It adds default functions like a magic method call 
 * for all child classes
 *
 * @subpackage 	framework.classes
 * @since 		1.0.0
 */
class BookzoObject extends BookzoResponseData {
	
	/**
	 * Reference to the global App object
	 *
	 * @var 	App
	 * @since 	1.0.0
	 */
	protected $images;
	
	/**
	 * An alias for this object
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $alias;
	
	/**
	 * The seo url to this object
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $link;
		
	/**
	 * An holder for the periods binded to this object
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $periods;
	
	
	/**
	 * An holder for the periods binded to this object
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $arrivals = array();
		
	/**
	 * An holder for the periods binded to this object
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $features;
	
	/**
	 * An holder for the periods binded to this object
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $favourite;
	
	/**
	 * An holder for the periods binded to this object
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $groupname;
	
	/**
	 * An holder for the periods binded to this object
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $group;
	
	/**
	 * An holder for the periods binded to this object
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $searchresult;
	
	protected $gallery;
	
	protected $widgets;
	
	protected $map;
	
	protected $lat;
	
	protected $lng;
	
	protected $booked;
	
	protected $maxmonth;
	
	protected $arrangements;
	
	/**
	 * An holder for the periods binded to this object
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	public $prices = array();
	
		
	/**
	 * Bind data to object.
	 * Override for parent class so we can set our own data
	 *
	 * @param   object  $item  The data object
	 * @return  void
	 * @access	proteced
	 * @since   1.0.0
	 */
	protected function bind($item)
	{		
		// set general properties
		$this->setProperties((array) $item, array('features'));	

		// try binding features
		$features = array();
		if ( isset($item->features) && count($item->features) ) 
		{		
			foreach ( $item->features as $feature ) {
				$features[$feature->featureid] = $feature->value;
			}
		}
		$this->set('features', $features);	

		$geocode = $this->get('geocode');
		if ( !empty($geocode) ) 
		{
			$codes = explode(',', $geocode);
			if ( count($codes) == 2 ) {
				$this->set('lat', $codes[0]);
				$this->set('lng', $codes[1]);
			}
		}
		
		// set alias
		$this->set('alias', strtolower(str_replace(' ', '-', trim($this->translate('name')))));

		// add periods
		$this->periods = null;	
	}
	
	/**
	 * Get the object introtext
	 * If not set and altdescription is set we use the default description
	 *
	 * @param   none
	 * @return  $return	string	the text for the introtext
	 * @access	proteced
	 * @since   1.0.0
	 * @modded 	1.0.1 - added suffix support
	 * @modded 	1.4.2 - added param override support
	 */	
	public function getShortDescription( $length = 0, BRegistry $params = null )
	{
		$return 	= $this->translate('shortdescription');
		$params 	= $this->app->registry->create($params);
		
		// no introtext then use default decription
		if ( (!strlen($return) && $params->get('altdescription', $this->app->config->get('template.altdescription', 1))) || $params->get('descriptionshort', 0) ) 
		{
			## NOTE - override length for now (texelduinen)
			$length				= $length 
								? $length 
								: (int) $params->get('altdescriptionlength', $this->app->config->get('template.altdescriptionlength', 200));
			
			$description 		= strlen($return) ? $return : $this->translate('description');
			$return 			= substr($description, 0, $length);
			$suffix 			= $params->get('altdescriptionsuffix', $this->app->config->get('template.altdescriptionsuffix', '...'));
			
			if ( strlen($return) < strlen($description) && strlen(trim($suffix))  ) {
				$return .= ' ' . $suffix;
			}
		}
		
		return $return;
	}
	
	public function getMetaDesc( $length = 160 )
	{
		$desc = $this->getShortDescription();
		
		return substr($desc, 0, $length);
	}
	
	/**
	 * Get object link
	 * We only build link if required 
	 *
	 * @param   none
	 * @return  link	string	the link for the object
	 * @access	proteced
	 * @since   1.0.0
	 * @modded	1.4.2 added extra param
	 */	
	public function getLink( $route = true, $addbase = false, $extra = array() )
	{
		if ( is_null( $this->link ) ) 
		{
			$id	  		= $this->get('id');
			$alias 		= $this->get('alias');
			$groupid 	= $this->get('groupid');
			
			$this->set('link', 'layout=object&id='.$id.':'.$alias. '&groupid='.$groupid);
		}
		
		$link = $this->link;
		
		if ( count($extra) ) {
			foreach ( $extra as $k => $v ) {
				$link .= '&'.$k . '=' . $v;
			}
		}
		
		if ( $route ) {
			return $this->app->route->route( $link, true, $addbase );
		}
		
		return $link;
	}
		
	/**
	 * Get object group
	 * We only get group if required 
	 *
	 * @param   none
	 * @return  object	group	a reference to the bookzo group
	 * @access	proteced
	 * @since   1.0.0
	 */
	public function getGroup()
	{
		if ( is_null( $this->group ) ) 
		{
			if ( $this->get('groupid') ) {
				$this->set('group', $this->app->api->getGroups($this->get('groupid')));
			}
			else {
				$this->group = false;
			}
		}
		
		return $this->group;
	}
	
	/**
	 * Get object groupname
	 * We only get groupname if required 
	 *
	 * @param   none
	 * @return  string	name  the name of the group
	 * @access	proteced
	 * @since   1.0.0
	 */
	public function getGroupName()
	{
		if ( $group = $this->getGroup() ) 
		{
			return $group->translate('name');
		}
		
		return '';
	}
	
	/**
	 * Get object group
	 * We only get group if required 
	 *
	 * @param   none
	 * @return  object	group	a reference to the bookzo group
	 * @access	proteced
	 * @since   1.0.0
	 */
	public function addPrice( $price )
	{
		$price = $this->app->price->create($price);
		
		$this->prices[$price->arrival.$price->departure.$price->arrangementid] = $price;
		ksort($this->prices);
	}
	
	/**
	 * See if object has images
	 *
	 * @param   none
	 * @return  boolean		true if images found, false otherwise
	 * @access	public
	 * @since   1.0.0
	 */
	public function hasImages()
	{
		return count($this->images);
	}
	
	public function getPrice($arrival = null, $departure = null)
	{
		$aid = 0;
		
		if ( !$arrival || !$departure ) {
			$values 		= $this->app->registry->create( $this->app->session->get('bookzo.search.query') );
			$arrival 		= $values->get('arrival');
			$departure 		= $values->get('departure');
			$aid		 	= $values->get('aid', 0);
		}
		
		$return = isset($this->prices[$arrival.$departure.$aid]) ? $this->app->registry->create($this->prices[$arrival.$departure.$aid]) : false;
		
		return $return;
	}
	
	public function getFirstPrice()
	{
		if ( count($this->prices) ) {
			return $this->app->array->first($this->prices);
		}
		
		return false;
	}
	
	/**
	 * See if object has prices
	 *
	 * @param   none
	 * @return  boolean		true if prices found, false otherwise
	 * @access	public
	 * @since   1.0.0
	 *
	 * DEPRECATED since 1.2.0 use getPrice instead
	 */
	public function hasPrice()
	{
		return $this->getPrice();
	}
	
	/**
	 * See if object has prices
	 *
	 * @param   none
	 * @return  boolean		true if prices found, false otherwise
	 * @access	public
	 * @since   1.0.0
	 *
	 * DEPRECATED since 1.2.0 not used anymore in code
	 */
	public function hasPriceList()
	{
		return;
	}
	
	/**
	 * Method to find the lowest price for this object
	 *
	 * @param   none
	 * @return  boolean		true if images found, false otherwise
	 * @access	public
	 * @since   1.0.0
	 */
	public function price($arrival = null, $departure = null, $format = 'long')
	{
		$found 	  = $this->getPrice($arrival, $departure);
		
		if ( !$found ) {
			return;
		}
		
		// override some params based on specific page
		$params = $this->app->registry->create();
		$params->set('pricesicon', $this->app->config->get('template.'.($this->app->input->get('layout') == 'object' ? 'detail' : 'teaser' ).'_pricesicon'));
		
		return $this->app->price->render($found, $format, $params);
	}
	
	public function getFavouriteButton($parentdiv)
	{
		return $this->app->bookzo->favourites->button($this->id, $parentdiv);
	}
	
	/**
	 * Get a list of possible periode prices for this object
	 *
	 * @param   none
	 * @return  array		a list of prices
	 * @access	public
	 * @since   1.0.0
	 */
	public function getPeriods( $persons = 2, $pets = 0, $arrangement = 0 )
	{	
		$key = $persons . '-'. $pets . '-' . $arrangement;
		
		if ( !isset($this->periods[$key]) ) 
		{
			$maxmonth	   = $this->getMaxMonth();
			$from		   = $this->app->date->today()->format('Y-m-d');
			$until		   = $this->app->date->create($maxmonth)->format('Y-m-t');
			$objectid	   = $this->app->config->get('bookzo.bookpergroup', 0) ? 0 : $this->id;
			$group		   = $this->app->config->get('bookzo.bookpergroup', 0) ? $this->get('groupid') : 0;
			$permutated	   = 1;
					
			$results	   = $this->app->api->getPrices( compact('from','until','persons','pets','objectid','group','arrangement','permutated') );
			$prices		   = array();
			
			foreach ( $results as $result ) {
				$price = $this->app->price->create($result, array('persons' => $persons, 'pets' => $pets));				
				$prices[$price->key] = $price;
			}
		
			ksort($prices);
			
			$this->periods[$key] = $prices;
		}
		
		return $this->periods[$key];
	}
	
	public function getPossibleArrivals( $persons = 2, $pets = 0, $arrangement = 0 )
	{	
		$key = $persons . '-'. $pets . '-' . $arrangement;
		
		if ( !isset($this->arrivals[$key]) ) 
		{
			$maxmonth	   = $this->getMaxMonth();
			$from		   = $this->app->date->today()->format('Y-m-d');
			$until		   = $this->app->date->create($maxmonth)->format('Y-m-t');
			$object		   = $this->app->config->get('bookzo.bookpergroup', 0) ? 0 : $this->id;
			$group		   = $this->app->config->get('bookzo.bookpergroup', 0) ? $this->get('groupid') : 0;
			
			$this->arrivals[$key] = $this->app->api->getArrivals( compact('from','until','group','persons','pets','object','arrangement') );
		}
		
		return $this->arrivals[$key];
	}
	
	public function getBooked()
	{	
		if ( is_null($this->booked) ) 
		{
			$maxmonth	= $this->getMaxMonth();
			$from		= $this->app->date->create()->format('Y-m-01');
			$until		= $this->app->date->create($maxmonth)->format('Y-m-t');
			$objectid	= $this->id;

			$this->booked  = $this->app->api->getBooked( compact('objectid', 'from', 'until') );
		}
		
		return $this->booked;
	}
	
	public function getMaxMonth()
	{	
		if ( is_null($this->maxmonth) ) 
		{
			$groupid = $this->get('groupid');
			$months  = $this->app->api->getAvailableMonths( compact('groupid') );
			$this->maxmonth = array_pop($months);
		
		}
		
		return $this->maxmonth;
	}
	
	/**
	 * Thumbnail return the first image found
	 *
	 * @param   none
	 * @return  string	the image html
	 * @access	public
	 * @since   1.0.0
	 */
	public function image( $type = 'medium', $fulltag = false )
	{
		$return = '';
		$type 	= $type == 'large' || $type == 'small' ? $type : 'medium';
		
		if ( $this->hasImages() ) 
		{
			$type 	= 'image'.strtolower($type);
			$src 	= $this->images[0];
			
			if ( isset($src->$type) ) {
				$return = $src->$type;
			}
		}
		
		if ( $return && $fulltag ) {
			return '<img src="'.$return.'" />';
		}
		
		return $return;
	}
	
	public function button( $type, $link = null, $text = null, $class = '', $page = '', $data = '', $dates = null )
	{
		$arrival 	= '';
		$departure 	= '';
		
		// can we show a book button
		if ( $type == 'book' ) 
		{
			$values 	= $this->app->registry->create( $this->app->session->get('bookzo.search.query') );
			$arrival 		= $values->get('arrival', false);
			$departure 		= $values->get('departure', false);
			$persons 		= $values->get('persons', false);
			$arrangement 	= $values->get('aid', false);
			
			if ( !$arrival || !$departure ) {
				return '';
			}			
		}
		
		if ( $type == 'bookdiscount' ) {
			$arrival 	= is_array($dates) && isset($dates['arrival']) ? $dates['arrival'] : false;
			$departure 	= is_array($dates) && isset($dates['departure']) ? $dates['departure'] : false;
			$persons	= false;
			
			if ( !$arrival || !$departure ) {
				return '';
			}
		}
	
		
		switch ( $type ) {
			
			default:
				$type = 'default';
				$link = empty($link) ? $this->get('link') : $link;
				$text = empty($text) ? BText::_('MORE_INFO') : $text;
				break;
				
				
			case 'readmore' :
				$link = $this->get('link');
				$text = empty($text) ? BText::_('MORE_INFO') : $text;
				break;
				
			case 'discounts' :
				$link = '';
				$text = empty($text) ? BText::_('MORE_DISCOUNTS') : $text;
				break;
			
			case 'bookdiscount':
			case 'book' :
				
				$type 	   = 'book';	
				$link 	   = $this->app->route->route('layout=book&task=book.request&object='. $this->get('id') .'&arrival='.$arrival. '&departure='.$departure . ( $persons ?  '&persons='.$persons : '') . ($arrangement ? '&arrangement='.$arrangement : ''));
				
				if ( empty($text) ) {
					if ( $this->app->config->get('template.full_show_dates', 0) && $page == 'full' ) {
						$starttext = $this->app->date->create($arrival)->format('DATE_FORMAT_SHORT');
						$endtext   = $this->app->date->create($departure)->format('DATE_FORMAT_SHORT');
						$text 	   = BText::sprintf('BUTTON_BOOK_DATES', $starttext, $endtext);
					}
					else {
						$text = BText::_('BUTTON_BOOK');
					}
				}

				break;
		}
		
		return $this->app->renderer->partial()->render('button.'.$type, compact('link', 'text', 'class', 'data', 'page', 'arrival', 'departure'));
	}
	
	public function widget($name, $params = array(), $render = true)
	{
		// create param object
		$params = $this->app->registry->create($params);
		$key	= $name . $params->clean()->createkey();
		
		if ( !isset($this->widgets[$key]) ) 
		{
			switch ( $name ) 
			{
				// gallery widget
				case 'gallery' :
				
					if ( $widget = $this->app->widget->get('gallery') ) 
					{						
						foreach ( $this->images as $image ) {
							
							$imagetype = $params->get('images', 'large') == 'medium' ? 'imagemedium' : 'imagelarge';
							$lightbox  = $params->get('lightbox', 0) ? $image->imagelarge : null;
							
							$widget->addImage( $image->{$imagetype}, $image->imagesmall, (isset($image->name) ? $image->name : ''), $lightbox);
						}
					}
					
					break;
				
				// map widget
				case 'map' :
				
					if ( $widget = $this->app->widget->get('map') ) 
					{
						$lat = $this->get('lat');
						$lng = $this->get('lng');
						
						if ( !$lat || !$lng ) {
							$widget->addAddress($this->get('address'), $this->get('postalcode'), $this->get('city'));
						}
						else {
							$widget->addMarker($lat, $lng);
						}
					}
					
					break;
				
				## NOTE - these look a lot alike
				// pricelist widget
				case 'pricelist' 	:
				case 'features' 	:
				case 'calendar' 	:
				
					$widget = $this->app->widget->get($name);
										
					break;
					
				// default
				default : 
				
					return '';				
			}
			
			// add parameters
			if ( $widget ) {
				$widget->setParams($params);
				$widget->setParam('id', $this->get('id'));
			}
			
			// add to internal cache
			$this->widgets[$key] = $widget;
		}
		
		// return only content if render is true
		if ( $render && $this->widgets[$key] !== false ) {
			return $this->widgets[$key]->render();
		}
			
		// return complete widget
		return $this->widgets[$key];
	}
	
	public function getFeatures()
	{		
		return $this->features;
	}
	
	public function getFeature($id, $default = null, $type = 'null')
	{
		$features = $this->features;
		$return   =  isset($features[$id]) ? $features[$id] : $default; 
		
		switch ( $type ) {
			default: return $return;
			case 'string'	: 	return (string) $return;
			case 'int'   	: 	if (strtolower($return) == 'true' ) { return 1; }
							  	if (strtolower($return) == 'false' ) { return 0; }
								return (int) $return;
			case 'boolean'  :	if (strtolower($return) == 'true' ) { return true; }
							  	if (strtolower($return) == 'false' ) { return false; }
								return (bool) $return;
		}
	}
	
	public function getPets()
	{
		if ( $this->get('petsallowed') == 'true' )
		{
			return (int) $this->get('numberofpetsallowed');
		}
		
		return false;
	}
	
	public function getPersons($return = 2)
	{
		if ( $return < $this->get('minguestcount') ) {
			$return = $this->get('minguestcount');
		}
		
		if ( $return > $this->get('numberofbeds') ) {
			$return = $this->get('numberofbeds');
		}
		
		return $return;
	}
	
	/* 
	 * Check to see if object has a certain feature value
	 *
	 * @param 	$id		int		The id of the feature to check
	 * @param	$value	string	The value the feature shoul be
	 *
	 * @return	boolean
	 * @since	1.0.0
	 */
	public function hasFeature($id, $value = null)
	{
		$featuresvalues = $this->getFeatures();
		
		if ( array_key_exists( (int) $id, $featuresvalues) )
		{
			if ( $value ) {
				return strtolower($value) == strtolower($featuresvalues[$id]);
			}
			else {
				$feature = $this->app->api->getFeatures($id);
				
				if ( $feature->get('isboolean') ) {
					return strtolower($featuresvalues[$id]) == 'true';				
				}
				else if ( (int) $featuresvalues[$id] == $featuresvalues[$id] ) {
					return (int) $featuresvalues[$id] != 0;
				}
				
				return true;
			}
		}
		
		return false;
	}
		
	public function getFavourite()
	{
		if ( is_null($this->favourite) ) {
			$this->favourite = $this->app->bookzo->favourites->isFavourite($this->id);
		}
		
		return $this->favourite;
	}
	
	/* 
	 * Check to see if object has a certain feature value
	 *
	 * @param 	$id		int		The id of the feature to check
	 * @param	$value	string	The value the feature shoul be
	 *
	 * @return	boolean
	 * @since	1.0.0
	 */
	public function getArrangements($id = null, $keysonly = false)
	{
		if ( is_null($this->arrangements) ) {
			$this->arrangements = array();
			$arrangements 		= $this->app->api->getArrangements();
			foreach ( $arrangements as $arrangement ) {
				if ( $arrangement->hasObject($this->id) ) {
					$this->arrangements[$arrangement->get('id')] = $arrangement;		
				}
			}
		}
		
		if ( $id ) {
			return isset($this->arrangements[$id]) ? ($keysonly ? $id : $this->arrangements[$id]) : false;
		}
		
		return ($keysonly ? array_keys($this->arrangements) : $this->arrangements);
	}
}
