<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

/**
 * Bz data class.
 *
 * This class acts as parent for all data classes
 * It adds default functions like a magic method call 
 * for all child classes
 *
 * @subpackage 	framework.classes
 * @since 		1.0.0
 */
class BookzoProspect extends BookzoResponseData {

	protected $cache = array();
	
	/**
	 * Bind data to object.
	 * Override for parent class so we can set our own data
	 *
	 * @param   object  $item  The data object
	 * @return  void
	 * @access	proteced
	 * @since   1.0.0
	 */
	protected function bind($item)
	{	
		parent::bind($item);
	}
	
	public function isValid()
	{
		// we must have at least some values
		if ( !isset($this->total) || !( (int) $this->total) || !isset($this->arrival) || !isset($this->departure) ) {
			return false;
		}
		
		return true;
	}
	
	public function getPriceElements()
	{
		$return = array();
		
		if ( isset($this->reservationlines) ) 
		{
			foreach ( $this->reservationlines as $line )
			{	
				if ( isset($line->amount) && $line->amount > 0 ) { 	
					$line->pricestring = number_format($line->total, 2, ',', '.');
					
					if ( isset($line->action) && (int) $line->action )  {
						$action = $this->app->api->getActions( false, (int) $line->action );
						
						if ( $action ) {
							$line->action = $action->get('description', $action->get('name'));
						}
					}
					
					$return[] = $line;
				}
			}
		}
		
		return $return;
	}
	
	public function getTotalPrice($asstring = false)
	{
		$return = isset($this->total) ? $this->total : 0;
		
		if ( $asstring ) {
			$return = number_format($return, 2, ',', '.');
		}
		
		return $return;
	}
	
	public function getBookableItems()
	{
		$return = array();
		
		if ( isset($this->optionalcosts) ) 
		{
			foreach ( $this->optionalcosts as $item )
			{		
				$item->pricestring = number_format($item->price, 2, ',', '.');
				$return[] = $item;
			}
		}
		
		return $return;
	}
	
	public function getArrangements()
	{
		if ( !isset($this->cache['arrangements']) ) 
		{
			$return = array();
			
			if ( isset($this->possiblearrangements) && is_array($this->possiblearrangements) && count($this->possiblearrangements) ) 
			{
				$return = $this->app->api->getArrangements($this->possiblearrangements);
			}
			
			$this->cache['arrangements'] = $return;
		}
		
		return $this->cache['arrangements'];
	}
	
	public function getReservationLineValue($type, $var, $default = null)
	{
		if ( isset($this->reservationlines) ) 
		{
			foreach ( $this->reservationlines as $line )
			{
				if ( strtolower($line->reservationlinetype) == strtolower($type) ) 
				{
					$var = strtolower($var);
					if ( isset($line->{$var}) ) {
						return $line->{$var};
					}
				}
			}
		}
		
		return $default;
	}
}