<?php
/**
 * @package   bookzo
 * @author    Invenda  http://www.invenda.nl
 * @copyright Copyright (C) Invenda
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

/**
 * Helper do deal with request variables. Wrapper for JRequest
 *
 * @package Framework.Helpers
 */
class BzHelperAdapter extends BHelper {

	/**
	 * Name of the wrapper class
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected $_class;

	/**
	 * Class Constructor
	 * 
	 * @param App $app A reference to the global App object
	 */
	public function __construct($app) {
		
		parent::__construct($app);

		$this->_class = BAdapter::getInstance($app);
	}
	
	/**
	 * Let the CMS redirect us to another page
	 *
	 * @params	 	$link	the page we are redirecting to
	 * @params		$status	the html page code. default = 303
	 * @return		void
	 * @access		public
	 * @since 		1.0.0
	 */
	public function redirect($link, $status = 302)
	{	
		$this->app->log->write('Redirecting', 'Redirecting to page ' . $link, '', 'tracking' );
		$this->_class->redirect($link, $status);
		
		return;
	}

	/**
	 * Map all the methods to the mapped class
	 *
	 * @param string $method The name of the method
	 * @param array $args The list of arguments to pass on to the method
	 *
	 * @return mixed The result of the call
	 *
	 * @see BInputFilter
	 *
	 * @since 1.0.0
	 */
    public function __call($method, $args) {
		return $this->_call(array($this->_class, $method), $args);
    }
}