<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

defined('_BOOKZO') or die;

/**
 * Bz data helper class.
 *
 * Helper for managing and retrieving paths
 * We can register paths.
 * When loading files we search paths top to bottom
 *
 * @subpackage 	framework.helpers
 * @since 		1.0.0
 */
class BzHelperConfig extends BHelper {

	public $config = null;
	
	public function __construct($app) 
	{
		parent::__construct($app);
		
		if ( is_null($this->config) ) 
		{
			// setup the config
			$this->config = new BRegistry();
			
			// try loading the correct file
			if ($this->app->filesystem->file->exists(BOOKZO_ROOT . '/config/bookzo.config')) 
			{
				$this->config->loadFile(BOOKZO_ROOT . '/config/bookzo.config');
			}
			
			$this->legacy();
		}
	}
	
	public function add( $new, $prefix = '')
	{
		$this->config->set($prefix, $new);
	}
	
	/**
	 * Merge a Registry object into this one
	 *
	 * @param   Registry  $source     Source Registry object to merge.
	 * @param   boolean   $recursive  True to support recursive merge the children values.
	 *
	 * @return  Registry  Return this object to support chaining.
	 *
	 * @since   1.0
	 */
	public function merge($source, $recursive = false)
	{
		$this->config->merge($source, $recursive);
		$this->legacy();

		return $this;
	}
	
	public function reset($config)
	{
		$this->config = $this->app->registry->create($config);
		$this->legacy();

		return $this;
	}
		
	/**
	 * Map all function to the config
	 */
	public function __call($method, $args) {
		return $this->_call(array($this->config, $method), $args);
	}
	
	/**
	 * Make a JSON string out of the config
	 *
	 * @param   none
	 * @return  String  A JSON serialized string
	 *
	 * @since   1.0
	 */
	public function __toString()
	{
		return (string) $this->config;
	}
	
	/**
	 * A Legacy method. 
	 * Added to make sure params are set correctly on updates if some change or no longer exist
	 *
	 * @return  Registry  Return this object to support chaining.
	 *
	 * @since   1.0
	 */
	protected function legacy()
	{		
		// since 1.3.2: you can choose to set pricecalculation aside instead of by default if months = 1
		if ( !$this->config->exists('widgets.calendar.pricecalculationaside')) {
			$this->config->set('widgets.calendar.pricecalculationaside', $this->config->get('widgets.calendar.months', 2) == 1 ? 1 : 0);
		}
		
		// since 1.3.2: you can choose to not show persons at all
		if ( !$this->config->exists('widgets.calendar.showpersons')) {
			$this->config->set('widgets.calendar.showpersons', $this->config->get('widgets.calendar.choosepersons', 0) == 1 ? 2 : 1);
		}
		
		// since 1.3.2: we choose a type of calendar
		if ( !$this->config->exists('widgets.calendar.type')) {
			switch ( $this->config->get('widgets.calendar.arrivalday', 2) ) 
			{
				default: $type = 'none'; break;
				case 1: case 2: $type = 'arrivals'; break;
				case 3: $type = 'type'; break;
			}
			$this->config->set('widgets.calendar.type', $type);
		}
		
		// since 1.3.2: we choose a action or not on calendar
		if ( !$this->config->exists('widgets.calendar.action')) {
			$this->config->set('widgets.calendar.action', $this->config->get('widgets.calendar.arrivalday', 2) >= 2 ? 1 : 0);
		}
		
		// since 1.3.2: we renamed some calendar values
		if ( !$this->config->exists('widgets.calendar.bookedbydefault')) {
			$this->config->set('widgets.calendar.bookedbydefault', $this->config->get('widgets.calendar.calendar_bookedbydefault', 1));
		}
		if ( !$this->config->exists('widgets.calendar.pricestyle')) {
			$this->config->set('widgets.calendar.pricestyle', $this->config->get('widgets.calendar.calendar_pricestyle', ''));
		}
		if ( !$this->config->exists('widgets.calendar.maxmonths')) {
			$this->config->set('widgets.calendar.maxmonths', $this->config->get('widgets.calendar.max', 12));
		}
		
		return $this;
	}
}