<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

defined('_BOOKZO') or die;

/**
 * Bz data helper class.
 *
 * Helper for managing and retrieving paths
 * We can register paths.
 * When loading files we search paths top to bottom
 *
 * @subpackage 	framework.helpers
 * @since 		1.0.0
 */
class BzHelperPrice extends BHelper {

	public function create($price, $addons = array()) 
	{
		$price 		= $this->app->registry->create($price);
		
		if ( $price->get('key') ) {
			return $price->toObject();
		}
		
		$addons 	= $this->app->registry->create($addons);
		$return   	= $this->app->registry->create();
		
		$return->set('key',					'');
		$return->set('priceid', 			$price->get('priceid', 				$addons->get('priceid', 0)));
		$return->set('actionid', 			$price->get('actionid', 			$addons->get('actionid', 0)));
		$return->set('arrival', 			$price->get('arrival', 				$addons->get('arrival')));
		$return->set('departure', 			$price->get('departure',			$addons->get('departure')));
		$return->set('originalname',		$price->get('originalname',			$addons->get('originalname')));
		$return->set('periodname', 			'');
		$return->set('specialname', 		'');
		$return->set('price', 		 		$price->get('price', 				0));
		$return->set('extracosts', 			$price->get('extracosts', 			0));
		$return->set('pricebeforediscount', $price->get('pricebeforediscount',  0));
		$return->set('total', 		 		$price->get('total',				0));
		$return->set('arrangementid', 		$price->get('arrangementid',  		0));
		$return->set('nights',	 			0);
		$return->set('days',	 			0);
		$return->set('week', 				0);
		$return->set('midweek',				0);
		$return->set('weekend',				0);
		$return->set('minnights',	 		0);
		
		if ( $persons = $addons->get('persons') ) {
			$return->set('persons', $persons);
		}
		if ( $pets = $addons->get('pets') ) {
			$return->set('pets', $pets);
		}
		
		
		if ( $objectid = $price->get('objectid', $addons->get('objectid')) ) {
			$return->set('objectid' , $objectid);
		}
		
		if ( $arrival = $return->get('arrival') ) {
			$return->set('arrival', 	 $this->app->date->create($arrival)->format('Y-m-d'));
			$return->set('arrivalstamp', $this->app->date->create($arrival)->toUnix());
		}
		
		if ( $departure = $return->get('departure') ) {
			$return->set('departure', 	 $this->app->date->create($departure)->format('Y-m-d'));
			$return->set('departurestamp', $this->app->date->create($departure)->toUnix());
		}
		
		// set key
		$key = $return->get('arrival') . $return->get('departure');
		
		// set nr of nights
		$return->set('nights', 	$this->app->date->diff($return->get('arrival'), $return->get('departure')));
		$return->set('days', 	$return->get('nights') + 1);
				
		// discount before can not be lower
		if ( $return->get('pricebeforediscount') < $return->get('price') ) {
			$return->set('pricebeforediscount', $return->get('price'));
		}
		
		// total can not be lower
		if ( $return->get('total') < $return->get('price') ) {
			$return->set('total', $return->get('price'));
		}
		
		// calculate totalbeforediscount
		$return->set('totalbeforediscount', (float) $return->get('pricebeforediscount') + (float) $return->get('extracosts'));
		
		// calculate discount
		$return->set('discount', (float) $return->get('pricebeforediscount') - (float) $return->get('price'));
					
		// add pricename if applied
		if ( ($priceid = $return->get('priceid')) && substr($priceid, 0, 1) != 'a' && ($pricedef = $this->app->api->getPriceDefinitions($priceid)) ) {
			$day 	= date('D', strtotime($return->get('arrival')));
			$return->set('originalname', $pricedef->get('name'));
			$return->set('minnights',  $pricedef->get('minnumberofnights'.strtolower($day), $pricedef->get('minnumberofnights', 0)));
		}
		else {
			$return->set('priceid', 'n'. $return->get('nights'));
		}
				
		// add actionname if applied
		if ( ($actionid = $return->get('actionid')) && ($action = $this->app->api->getActions(false, $actionid)) ) {
			$return->set('actionname', $this->app->config->get('template.discounts_name_type', 'name') == 'name' 
											? $action->get('name') 
											: $action->get('description', $action->get('name'))
			); 
		}
		
		// add arrangementname if applied
		if ( ($arrangementid = $return->get('arrangementid')) && ($arrangement = $this->app->api->getArrangements($arrangementid)) ) {
			$return->set('originalname', $arrangement->get('name'));
			$return->set('priceid',  	 'a' . $arrangement->get('id'));
		}
				
		// get the names of the period
		$nights 	= $return->get('nights');
		$startday 	= $this->app->date->create($arrival)->format('w');
		$return->set('specialname',		$this->getPriceName($return->get('arrival'), $return->get('departure'), null, true));
		$return->set('periodname', 		$this->getPriceName($return->get('arrival'), $return->get('departure'), false));
		$return->set('week', 			!($nights%7));
		$return->set('midweek', 		(($nights == 4 || $nights == 5) && $startday == 1));
		$return->set('weekend', 		(($nights == 3 || $nights == 4) && $startday == 5));
		
		// add extracostnames
		$extracosts = $price->get('costsspecification', array());
		foreach ( $extracosts as $extracost ) {
			if ( $extracost->isarrangement ) {
				if ( $arrangement = $this->app->api->getArrangements($extracost->id) ) {
					$extracost->name = $arrangement->get('name');
					$return->set('arrangementid', $extracost->id);
				}
			}
			else if ( $cost = $this->app->api->getExtraCosts($extracost->id) ) {
				$extracost->name = $cost->get('name');
			}
			else {
				$extracost->name = '';
			}
		}
		
		$return->set('costsspecification', 	$extracosts);	
		$return->set('key', $key . $return->get('arrangementid'));
		
		return $return->toObject();
	}
	
	public function getPriceName($arrival, $departure, $change = null, $returnempty = false)
	{
		$arrival 	= $this->app->date->create($arrival)->format('Y-m-d');
		$departure 	= $this->app->date->create($departure)->format('Y-m-d');
		$nights 	= $this->app->date->diff($arrival, $departure);
		$days		= $nights + 1;
		$return 	= '';
		
		if ( is_null($change) ) {
			$change = $this->app->config->get('template.durationchange', 1);
		}
				
		if ( $change ) 
		{
			$startday 	= $this->app->date->create($arrival)->format('w');
			
			if ( ($nights == 2 || $nights == 3) && $startday == 5) {
				$return = BText::_('PERIOD_TITLE_WEEKEND'.($nights == 3 ? '_LONG' : ''));
			}
			elseif ( ($nights == 4 || $nights == 5 ) && $startday == 1 ) {
				$return = BText::_('PERIOD_TITLE_MIDWEEK'.($nights == 5 ? '_LONG' : ''));
			}
			elseif ( !($nights%7) ) {
				$return = BText::plural('PERIOD_TITLE_WEEKS',($nights/7) );
			}
		}
		
		if ( !$return && !$returnempty ) {
			$nametype	= $this->app->config->get('template.durationstyle', 'days');
			
			if ( $nametype == 'days' ) {
				$return = BText::plural('PERIOD_TITLE_DAYS', $days);
			}
			elseif ( $nametype == 'nights' ) {
				$return = BText::plural('PERIOD_TITLE_NIGHTS', $nights);
			}
			else {
				$return = BText::sprintf('PERIOD_TITLE_DAY'.($days == 1 ? '':'S').'_NIGHTS', $nights, $days);
			}
		}
		
		return $return;
		
	}
	
	public function render($price, $type = 'default', $params = null)
	{
		// create priceobject
		$params 	= $this->app->registry->create($params);
		$price 		= $this->app->registry->create( $this->create($price) );
				
		// lets render : )
		return $this->app->renderer->partial()->render('price.'.$type, compact('price', 'params'));
	}
	
	public function booklink($price = array(), $vars = array(), $xhtml = true)
	{	
		$price = $this->app->registry->create($price);
		$vars  = $this->app->registry->create($vars);

		$link = 'layout=book&task=book.request';
		
		
		if ( $objectid = $price->get('objectid', $vars->get('objectid'))) {	
			$link .= '&object='.$objectid;
		}
		if ( $arrival = $price->get('arrival', $vars->get('arrival'))) {
			$link .= '&arrival='.$arrival;
		}
		if ( $departure = $price->get('departure', $vars->get('departure'))) {
			$link .= '&departure='.$departure;
		}
		if ( $arrangementid = $price->get('arrangementid', $vars->get('arrangementid'))) {
			$link .= '&arrangement='.$arrangementid;
		}
		if ( $persons = $price->get('persons', $vars->get('persons'))) {
			$link .= '&persons='.$persons;
		}
		if ( $pets = $price->get('pets', $vars->get('pets'))) {
			$link .= '&pets='.$pets;
		}
		
		return $this->app->adapter->route($link, $xhtml);
	}
	
	public function calculation($price, $jsonencode = false, $params = null)
	{
		$price 	= $this->app->registry->create( $this->create($price) );
		$return	= $this->app->renderer->partial()->render('price.minicalculation', compact('price', 'params'));
		
		if ( $jsonencode ) {
			return $this->app->string->dataAttributePrepare($return);
		}
		
		return $return;
	}
	
	public function format($price, $params = null, $attr = '')
	{
		$params = $this->app->registry->create($params);
		$html 	= '';
		
		// show currency
		if ( ($currency = $params->get('pricecurrency', $this->app->config->get('template.pricecurrency'))) && !$params->get('hidecurrency') ) {
			$html .= '<em class="bz-currency">'. $currency . '</em> ';
		}
		
		// format price
		$price 	= (float) $price;
		$style	= $params->get('pricestyle', $this->app->config->get('template.pricestyle'));
		
		switch( $style ) {
			
			default:
			case 'always': 
					$price = number_format($price, 2, ',', '.');
					break;
			case 'round':
					$price = number_format(round($price), 0, ',', '.');
					break;
			case 'nozero':
			case 'zerochange':
					if ( round($price) == $price ) {
						$price = number_format(round($price), 0, ',', '.') . ( $style == 'zerochange' ? ',-' : '' );
					}
					else {
						$price = number_format($price, 2, ',', '.');
					}
					break;
		
		}
		
		return $html . '<span '.$attr.'>' . $price . '</span>';
	}
	
}