<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

defined('_BOOKZO') or die;

/**
 * Bz data helper class.
 *
 * Helper for managing and retrieving paths
 * We can register paths.
 * When loading files we search paths top to bottom
 *
 * @subpackage 	framework.helpers
 * @since 		1.0.0
 */
class BzHelperRoute extends BHelper {

	public $current;
	
	
	/**
	 * Class Constructor
	 * 
	 * @param App $app A reference to the global App object
	 */
	public function __construct($app) {
		
		parent::__construct($app);

		$current = array();
		$current['protocol'] = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' ? 'https' : 'http');
		$current['host']	 = $_SERVER['HTTP_HOST'];
		$current['path']	 = $_SERVER["PHP_SELF"];
		$current['query']	 = $_SERVER["QUERY_STRING"];
		$current['scheme']	 = $_SERVER["REQUEST_URI"];

		$this->current = $current;
		
	}
	
	public function get($type) 
	{
		switch ( $type ) {
			case 'full':
				return $this->current['protocol'] .'://'. $this->current['host'] . $this->current['scheme'];
			case 'page':
				return $this->current['scheme'];
			default:
				if (isset($this->current[$type]) ) {
					return $this->current[$type];
				}
				return '';
		}
	}
	
	/**
	 * Map all the methods to the adapter class
	 *
	 * @param string $method The name of the method
	 * @param array $args The list of arguments to pass on to the method
	 *
	 * @return mixed The result of the call
	 *
	 * @see BInputFilter
	 *
	 * @since 1.0.0
	 */
    public function __call($method, $args) {

		//return $this->_call(array($this->app->adapter, 'route'), $args);
		return $this->_call(array($this->app->adapter, $method), $args);
    }
}