<?php
/**
 * @package   bookzo
 * @author    Invenda  http://www.invenda.nl
 * @copyright Copyright (C) Invenda
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

/**
 * Helper do deal with request variables. Wrapper for JRequest
 *
 * @package Framework.Helpers
 */
class BzHelperUserState extends BHelper {

	/**
	 * Name of the wrapper class
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected $_session;

	/**
	 * Class Constructor
	 * 
	 * @param App $app A reference to the global App object
	 */
	public function __construct($app) {
		
		parent::__construct($app);

		$this->_session = $this->app->session;
	}
	
	/**
	 * Gets a user state.
	 *
	 * @param   string  $key      The path of the state.
	 * @param   mixed   $default  Optional default value, returned if the internal value is null.
	 *
	 * @return  mixed  The user state or null.
	 *
	 * @since   3.2
	 */
	public function get($key, $default = null)
	{
		$registry = $this->app->registry->create( $this->_session->get('bookzo.userstate') );
		
		return $registry->get($key, $default);
	}
	
	/**
	 * Sets the value of a user state variable.
	 *
	 * @param   string  $key    The path of the state.
	 * @param   string  $value  The value of the variable.
	 *
	 * @return  mixed  The previous state, if one existed.
	 *
	 * @since   3.2
	 */
	public function set($key, $value)
	{
		$registry   = $this->app->registry->create( $this->_session->get('bookzo.userstate') );
		$return 	= $registry->set($key, $value);
		$this->_session->set('bookzo.userstate', $registry->toString());
			
		return $return;
	}
	
	/**
	 * Gets the value of a user state variable.
	 *
	 * @param   string  $key      The key of the user state variable.
	 * @param   string  $request  The name of the variable passed in a request.
	 * @param   string  $default  The default value for the variable if not found. Optional.
	 * @param   string  $type     Filter for the variable, for valid values see {@link JFilterInput::clean()}. Optional.
	 *
	 * @return  object  The request user state.
	 *
	 * @since   3.2
	 */
	public function fromRequest($key, $request, $default = null, $type = 'none')
	{
		$cur_state = $this->get($key, $default);
		$new_state = $this->app->input->get($request, null, $type);

		// Save the new value only if it was set in this request.
		if ($new_state !== null)
		{
			$this->set($key, $new_state);
		}
		else
		{
			$new_state = $cur_state;
		}

		return $new_state;
	}

	/**
	 * Map all the methods to the mapped class
	 *
	 * @param string $method The name of the method
	 * @param array $args The list of arguments to pass on to the method
	 *
	 * @return mixed The result of the call
	 *
	 * @see BInputFilter
	 *
	 * @since 1.0.0
	 */
    public function __call($method, $args) {
		return $this->_call(array($this->_class, 'session' . ucfirst($method)), $args);
    }
}