<?php
/**
 * @package   bookzo
 * @author    Invenda  http://www.invenda.nl
 * @copyright Copyright (C) Invenda
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

/**
 * Base View class
 *
 * @package Framework.Classes
 */
class BzModelArrangement extends BModelList 
{	
	protected $query;
	
	public function getArrangement()
	{		
		$id 	 		= $this->state->get('aid');
		$arrangement	= $this->app->api->getArrangements($id);

		return $arrangement;
	}
	
	public function getItems()
	{	
		// try loading from cache
		if ( isset($this->cache['items']) ) {
			return $this->cache['items'];
		}
		
		// get search data
		$items				= array();
		$arrival 			= $this->getState('arrival');
		$departure 			= $this->getState('departure');
		$persons			= $this->getState('persons');
		$pets				= $this->getState('pets');
		$group				= $this->getState('group');
		$arrangement		= $this->getState('aid');
				
		// do we have the query in session cache
		$found	 	= $this->app->api->getFreeObjects(compact('arrival', 'departure', 'group', 'persons', 'pets', 'arrangement'));				
		$this->app->session->set('bookzo.search.results', $found);

		// get ids found
		$foundids	= array_keys($found);

		// if found find matching objects
		if ( count($foundids) ) 
		{	
			$objects	= $this->app->api->getObjects($foundids);

			// add prices to the objects
			foreach ( $objects as $id => $object ) {
				$object->addPrice($found[$id]->price);			
				$object->set('searchresult', true);
			}

			$items = $objects;
		}
		
		// group and filter results 
		$items = $this->app->group->group($items);
		$items = $this->app->bookzo->filters->addObject($items)->getFiltered();

		// sort the items as requested
		$this->order($items);

		// update cache
		$this->cache['items'] = $items;
		
		return $this->cache['items'];
	}
	
	public function buildSearchQuery()
	{
		if ( is_null($this->query) ) 
		{
			// get old query
			$old 			= $this->app->registry->create( $this->app->session->get('bookzo.search.query', array()) )->clean(true);
			
			// did we come from another page
			$aid			= $this->app->input->getInt('aid');
			$isnew			= $aid != $old->get('aid') || $old->get('type') != 'arrangement';
			
			// build new query
			$new 			= $this->app->registry->create();
			$new->set('type', 'arrangement');
			$group  		= $new->set('group', 		$this->app->input->getInt('group', 		($isnew ? '' : $old->get('group'))));
			$arrival 		= $new->set('arrival', 		$this->app->input->getCmd('arrival', 	($isnew ? '' : $old->get('arrival'))));
			$departure		= $new->set('departure', 	$this->app->input->getCmd('departure', 	($isnew ? '' : $old->get('departure'))));
			$month			= $new->set('month', 		$this->app->input->getCmd('month', 		($isnew ? '' : $old->get('month'))));
			$persons		= $new->set('persons', 		$this->app->input->getInt('persons', 	($isnew ? 2  : $old->get('persons', 2))));
			$pets			= $new->set('pets', 		$this->app->input->getInt('pets', 		($isnew ? '' : $old->get('pets'))));
			$arrangement	= $new->set('aid', 			$aid);
												
			$new->clean(true);

			// do we need to continue or are we ok
			if ( $new->compare($old, array('type','group','arrival','departure','persons','pets','aid')) ) {
				$return = $old;
			}
			else {
				// store data in the session and reset
				$this->app->session->set('bookzo.pagetype', 'arrangement');
				$this->app->userstate->set('arrangement.limitstart', 0);
				$this->app->session->clear('bookzo.search.results');
				$this->app->session->clear('bookzo.calendar.values');
				$this->app->session->clear('bookzo.arrangement.filters');

				// reapply filters if they are set
				$filters = $this->app->input->get('filters', array(), 'array');
				if ( count($filters) ) {
					$this->app->session->set('bookzo.arrangement.filters', $filters);
				}

				// determine arrival
				if ( !$arrival ) 
				{
					$arrival = $new->set('arrival', $this->app->api->getArrivals( compact('arrangement', 'group', 'persons', 'pets'), true ));
				}

				// determine departure
				if ( !$departure || $arrival != $old->get('arrival') )
				{
					$departure = $new->set('departure', $this->app->api->getDepartures( compact('arrangement', 'arrival', 'group', 'persons', 'pets'), true ));
				}

				// also store real query data in the session
				$this->app->session->set('bookzo.search.query', $new->toArray() );
				
				$return = $new;
			}
			
			// remove month, duration and type (we don't need it in url)
			$return->remove(array('month', 'duration', 'type'))->clean(true);
		
			$this->query = $return;			
		}
		
		return $this->query;
	}
	
	/**
	 * Method to auto-populate the model state.
	 *
	 * This method should only be called once per instantiation and is designed
	 * to be called on the first call to the getState() method unless the model
	 * configuration flag to ignore the request is set.
	 *
	 * @return  void
	 *
	 * @note    Calling getState in this method will result in recursion.
	 * @since   12.2
	 */
	protected function populateState( $order = 'id', $dir = 'asc' )
	{
		// call parent populate for default settings
		parent::populateState( $this->app->config->get('template.arrangement_order'), $this->app->config->get('template.arrangement_order_dir') );
		
		$values = $this->buildSearchQuery();

		$this->state->set('arrival', 	 $values->get('arrival'));
		$this->state->set('departure', 	 $values->get('departure'));
		$this->state->set('persons', 	 $values->get('persons', 2));
		$this->state->set('pets', 		 (int) $values->get('pets', 0));
		$this->state->set('group', 		 (int) $values->get('group', 0));
		$this->state->set('aid', 		 (int) $this->app->input->get('aid', 0));
	}
}