<?php
/**
 * @package     Bookzo.Platform
 * @subpackage  Form
 *
 * @copyright   Copyright (C) 2005 - 2014 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('_BOOKZO') or die;

/**
 * Form Field class for the Joomla Platform.
 * Single check box field.
 * This is a boolean field with null for false and the specified option for true
 *
 * @package     Bookzo.Platform
 * @subpackage  Form
 * @link        http://www.w3.org/TR/html-markup/input.checkbox.html#input.checkbox
 * @see         BFormFieldCheckboxes
 * @since       11.1
 */
class BFormFieldArrangement extends BFormFieldSelect
{
		/**
	 * The form field type.
	 *
	 * @var    string
	 * @since  11.1
	 */
	protected $type = 'Terms';

	/**
	 * Method to get the field input markup.
	 * The checked element sets the field to selected.
	 *
	 * @return  string  The field input markup.
	 *
	 * @since   11.1
	 */
	protected function getInput()
	{
		// get basic stuff
		$app	   		= Bookzo::getInstance();
		$arrangements   = $this->getOptions();
						
		// do we need to do anything
		if ( !count($arrangements) ) {
			return;
		}
		
		// Initialize some field attributes.
		$class 		= !empty($this->class) 	? ' class="' . $this->class . '"' : '';
		$disabled 	= $this->disabled 		? ' disabled' : '';
		$html 	   	= array();
		
		// build html
		foreach ( $arrangements as $arrangement ) {
			$active = $arrangement->get('id') == $this->value;
			$data	= ' data-bz-updatearrangement="{value: '. ($active ? 0 : $arrangement->get('id')) .', id: \'' . $this->id .'\'}" ';
			$html[] = $app->renderer->partial()->render('arrangement.book', compact('arrangement', 'active', 'data', 'app') );
		}
		
		$html[] = '<input type="hidden" name="' . $this->name . '" id="' . $this->id . '" value="'
			. (int) $this->value . '"' . $class . $disabled . ' />';
		
		// return
		return implode("\n", $html);
	}

	/**
	 * Method to get the field input markup.
	 * The checked element sets the field to selected.
	 *
	 * @return  string  The field input markup.
	 *
	 * @since   11.1
	 */
	protected function getOptions()
	{
		$app 	 = Bookzo::getInstance();
		$options = array();
		$parents = parent::getOptions();
		
		foreach ( $parents as $parent ) {
			if ( $parent->value && ($arrangement = $app->api->getArrangements($parent->value)) ) {
				$options[$parent->value] = $arrangement;
			}				
		}

		return $options;
	}
}
