<?php
/**
 * @package   bookzo
 * @author    Invenda  http://www.invenda.nl
 * @copyright Copyright (C) Invenda
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

/**
 * Base View class
 *
 * @package Framework.Classes
 */
class BzModelSearch extends BModelList {
		
	public function getItems()
	{	
		// try loading from cache
		if ( isset($this->cache['items']) ) {
			return $this->cache['items'];
		}
		
		// get search data
		$items		= array();
		$searchword = $this->getState('searchword');
		
		// get items through group
		$items = $this->app->api->getObjects();
		
		// pull items through search if applicable
		if ( $searchword ) {
			$items = $this->app->search->filterByWord($items, $searchword);
		} 

		// group results if requested
		if ( $this->app->config->get('bookzo.bookpergroup', 0) ) {
			
			$tmp 		 = array();
			$foundgroups = array();
			foreach ( $items as $k => $item ) {
				if ( !in_array($item->get('groupid'), $foundgroups) ) {
					$foundgroups[] = $item->get('groupid');
					$tmp[$k] = $item;
				}
			}
			
			$items = $tmp;
		}
		
		// last but not least filter the items
		$items = $this->app->bookzo->filters->addObject($items)->getFiltered();

		// add the filtered elements to the session
		$this->app->session->set('bookzo.search.list', array_keys($items));
		
		// sort the items as requested
		$this->order($items);

		// update cache
		$this->cache['items'] = $items;
		
		return $this->cache['items'];
	}
	
	/**
	 * Method to auto-populate the model state.
	 *
	 * This method should only be called once per instantiation and is designed
	 * to be called on the first call to the getState() method unless the model
	 * configuration flag to ignore the request is set.
	 *
	 * @return  void
	 *
	 * @note    Calling getState in this method will result in recursion.
	 * @since   12.2
	 */
	protected function populateState( $order = 'id', $dir = 'asc' )
	{
		// call parent populate for default settings
		parent::populateState( $this->app->config->get('template.search_order'), $this->app->config->get('template.group_order_dir') );
		
		$this->state->set('searchword', $this->app->input->get('searchword', '')); 
		
		// do we have a pagination override
		$grouplimit = $this->app->config->get('template.search_pagination_limit', $this->app->config->get('template.pagination_limit'));
		if ( is_numeric($grouplimit) ) {
			$this->setState('list.limit', $grouplimit );
		}
	}
	
}