<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

defined('_BOOKZO') or die;

/**
 * Bz data helper class.
 *
 * Helper for managing and retrieving paths
 * We can register paths.
 * When loading files we search paths top to bottom
 *
 * @subpackage 	framework.helpers
 * @since 		1.0.0
 */
class BzWidgetCalendar extends BWidget {

	protected $calendar;
	
	protected $state;
	
	/**
	 * Render the module
	 *
	 * @param 	none
	 * @return 	mixed 	the html
	 * @since 	1.0.0
	 */
	public function render()
	{
		$id 	= (int) $this->params->get('id');
		
		// can we find this object
		if ( !$id || !$object = $this->app->api->getObjects($id) ) {
			return false;
		}
		
		// update state 
		$state 			= $this->getState(array('id' => $id));
		
		// get search params
		$objectmax 		= (int) $object->get('numberofbeds');
		$objectmin 		= (int) $object->get('minguestcount');
		$personssearch 	= (int) $state->get('persons', $object->get('persons', 2));
		$pets		 	= $state->get('pets', 0);
		$persons		= $personssearch < $objectmin ? $objectmin : ($personssearch > $objectmax ? $objectmax : $personssearch);	
		
		// set some params
		$dateformat			= $this->app->config->get('template.book_date_notation', 'DATE_FORMAT_DEFAULT');
		$this->params->set('dateformat', $dateformat);	
		$this->params->set('alwaysshowdiscount', true);	
		if ( $this->params->get('type') == 'none' ) {
			$this->params->set('showprices', 0);
		}
		
		// do we need to update
		if ( $persons > $personssearch ) {
			$state = $this->getState(array('persons' => $persons));
		}
		
		// setup the calendar
		$calendar 			= $this->setUpCalendar($object);

		// get stuff we need from calendar
		$params				= $this->params;
		$months 			= $calendar->render();
		$typelist			= $calendar->getTypeList();
		$activeprice	 	= $calendar->getActivePrice();
		$arrangement 		= false;
		if ( $aid = $state->get('aid') ) {
			$arrangement = $this->app->api->getArrangements($aid);
		}
		
		// get a pricecalculation (if requested)
		$pricecalulation 	= '';
		
		if ( $params->get('pricecalculation', 0) && $activeprice) 
		{
			$calcparams	= $params;
			$calcparams->set('pricestyle', 		$params->get('pricecalculationpricestyle'));
			$calcparams->set('hidecurrency', 	$params->get('pricecalculationhidecurrency'));
		
			$pricelist			= $this->params->get('showpricelist', 0)
								? $this->app->widget->get('pricelist')->setParam('id', $id)->setParam('overflow', false)->setParam('type', 'list')->setParam('aid', $aid)->render() 
								: false;
			$departurelist		= $calendar->getDepartureList();
			$alternatives		= $calendar->getAlternativePrices();
			$personlist			= $this->getPersonList($objectmin, $objectmax, $persons);	
			$pricecalculation	= $this->renderer->render('directbooking', 
										compact('calendar', 'activeprice', 'departurelist', 'personlist', 'dateformat', 'alternatives', 'calcparams', 'pricelist', 'arrangement')
									);	
		}
		else {
			$params->set('pricecalculation', 0);
		}
				
		$activemonth		= $calendar->getActiveMonth();
		$calendars		  	= $this->renderer->render('calendar', compact('months', 'params', 'activemonth'));	
		
		// render general template
		return  $this->renderer->render('default', compact('calendar', 'calendars', 'pricecalculation', 'params', 'typelist', 'arrangement'));	
	}



	public function setUpCalendar($object)
	{
		if ( is_null($this->calendar) ) 
		{
			$params	  = $this->params;
			$params->set('bookedbydefault', $this->app->config->get('bookzo.bookpergroup', 0) ? 1 : $params->get('bookedbydefault'));
			$state 	  = $this->getState();

			// initiate calendar and merge settings
			$calendar = new BookzoCalendar();
			$calendar->settings->merge($params);
			$calendar->search->merge($state, true);
												
			// add priceblocks to calendar	
			$prices  = $object->getPeriods($state->get('persons',$object->get('persons', 2)), $state->get('pets',0), $state->get('aid'));
			
			$calendar->addBlocks($prices);

			// do we want to close data
			if ( !$params->get('bookedbydefault', 1) ) {
				$booked = $object->getBooked();
				$calendar->addBlocks($object->getBooked());
			}
							
			// return the widget
			$this->calendar 	= $calendar;
		}
		
		return $this->calendar;
	}
	
		
	public function ajaxGetPrices()
	{
		$state   		= $this->getState();
		$persons 		= $this->app->input->get('persons');
		$oid 	 		= $this->app->input->get('oid');
		$pets	 		= $state->get('pets');
		$arrangement 	= $state->get('aid');
		
		// can we find this object
		if ( !$oid || !$object = $this->app->api->getObjects($oid) ) {
			throw new BWidgetException('Object not found');
		}
		
		$calendar 	= new BookzoCalendar();
		$blocks  	= $object->getPeriods($persons, $pets, $arrangement);
		$calendar->addBlocks($blocks);	

		$return 		= new stdClass();
		$return->link 	= $this->app->price->booklink(array(), array('objectid' => $oid, 'persons' => $persons, 'pets' => $pets, 'arrangementid' => $arrangement), false);
		$return->prices = $calendar->getPriceData();
		
		
		return $return;
	}
	
	/* 
	 * Build a list of min-max persons
	 *
	 * @param	int 	$min		minimal number of persons
	 * @param	int 	$max		maximum number of persons
	 * @param	date	$active		the active persons
	 */
	public function getPersonList($min, $max, $active)
	{
		$options = array();
		$attr	 = 'data-persons class="bz-input bz-size-small"';
		
		for ( $i = $min; $i <= $max; $i++ ) {
			$options[] = $this->app->html->select->option($i, $i);
		}
			
		return $this->app->html->select->genericlist('persons', $options, $active, $attr);
	}	
	
	/**
	 * Method to store and update values changed in calendar
	 * 
	 * @param 	string $name The name of the data class to retrieve 
	 * @return 	AppTable The table object
	 * @since 	1.0.0
	 */
	public function getState($vars = array())
	{
		// do we create a new state
		if ( is_null($this->state) || isset($vars['id']) ) 
		{
			// get search queries
			$state	 	 = $this->app->registry->create( $this->app->session->get('bookzo.calendar.values') );
			$searchstate = $this->app->registry->create( $this->app->session->get('bookzo.search.query') );
			$input		 = $this->app->input;

			// do we look at a new accommodation then reset
			if ( isset($vars['id']) && $state->get('id') != $vars['id'] ) {
				$state = $searchstate;
				$state->set('id', 	$vars['id']);
			}
			if ( $arrival 	= $input->get('arrival') ) {
				$state->set('arrival', 	$arrival);
			}
			if ( $departure = $input->get('departure') ) {
				$state->set('departure', 	$departure);
			}
			if ( $persons = $input->get('persons') ) {
				$state->set('persons', 	$persons);
			}
			if ( is_int( $input->get('pets') ) ) {
				$state->set('pets', $input->get('pets'));
			}
			if ( ($aid = $input->get('aid')) && strlen($aid) ) {
				$state->set('aid', (int) $aid);
			}
			if ( !$state->get('persons', 0) ) {
				$state->set('persons', 2);
			}
			
			$this->state = $state;
		}
		
		// update if needed
		foreach ( $vars as $k => $v ) {
			$this->state->set($k, $v);
		}
		
		// restore
		$this->app->session->set( 'bookzo.calendar.values', $this->state->toArray() );
		
		// return
		return $this->state;
	}
}