<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

defined('_BOOKZO') or die;

/**
 * Bz data helper class.
 *
 * Helper for managing and retrieving paths
 * We can register paths.
 * When loading files we search paths top to bottom
 *
 * @subpackage 	framework.helpers
 * @since 		1.0.0
 */
class BzWidgetDiscounts extends BWidget {

	/**
	 * Render the module
	 *
	 * @param 	none
	 * @return 	mixed 	the html
	 * @since 	1.0.0
	 */
	public function render()
	{	
		// get settings
		$params			= $this->params;
		$renderer		= $this->renderer;
		$objectids		= (array) $params->get('objects', array());
		$ids			= (array) $params->get('discounts', array());
		$exclude		= (array) $params->get('exclude', array());
		
		// get discounts and get only those wanted
		$actions		= $this->app->api->getActions(true);
		$actionids		= array_keys($actions);
		$actionids		= count($ids) 		? array_intersect($ids, $actionids) : $actionids;
		$actionids		= count($exclude) 	? array_diff($actionids, $exclude) 	: $actionids;
		
		// no discounts then return
		if ( !count($actionids) ) {
			return '';
		}
		
		## NOTE - ask jon to make getFreePeriods available for multiple objectids
		## That way we don't need to get all periods for all objects
		
		// get free periods for this discount
		$from		= $this->app->date->today('Y-m-d');
		$until		= $this->app->date->create()->add($this->app->config->get('actions.max', 2),'month')->format('Y-m-d');
		$discounts 	= $this->app->api->getPrices(array('actions' => $actionids, 'from' => $from, 'until' => $until), true);
		
		if ( !count($discounts) ) {
			return '';
		}
		
		// remove discounts without pricedrop
		foreach ( $discounts as $objectid => $prices ) 
		{
			// object must be in requested ids
			if ( count($objectids) && !in_array($objectid, $objectids) ) {
				unset($discounts[$objectid]);
				continue;
			}
			
			// loop prices removing non discounts
			foreach ( $prices as $k => $price ) {
				if ( !isset($price->pricebeforediscount) || $price->pricebeforediscount <= $price->price ) {
					unset($discounts[$objectid][$k]);
				}
			}
			
			// remove object if no discounts left
			if ( !count($discounts[$objectid]) ) {
				unset($discounts[$objectid]);
			}
			else {
				// we only need first
				$discounts[$objectid] = $this->app->array->first($discounts[$objectid]);
			}
		}
		
		// no discounts return
		if ( !count($discounts) ) {
			return '';
		}
				
		// get related object form api
		$itemids 	= array_keys($discounts);
		$items 		= $this->app->api->getObjects($itemids);
		
		// no items return
		if ( !count($items) ) {
			return '';
		}

		// add prices to each found object
		foreach ( $items as $item ) {
			$item->addPrice($discounts[$item->get('id')]);
		}
		
		// do we need to group
		if ( $this->app->config->get('bookzo.bookpergroup', 0) ) {	
			$tmp 		 = array();
			$foundgroups = array();
			foreach ( $items as $k => $item ) {
				if ( !in_array($item->get('groupid'), $foundgroups) ) {
					$foundgroups[] = $item->get('groupid');
					$tmp[$item->get('groupid')] = $item;
				}
			}
			$items = $tmp;
		}
		
		// do we need to slice
		$count 	= count($items);
		$max	= $params->get('max', 0);
		
		$items  = $this->randomize($params->createkey(), $items);
		
		if ( $max && $max < $count ) {
			$items = array_slice($items, 0, $max);
		}
		
		$objects = $items;
		
		
		// render general template
		return  $this->renderer->render('default', compact('objects', 'params', 'renderer', 'actions'));
	}
	
	protected function randomize($key, $items) 
	{
		$ids 		= array_keys($items);
		$key 		= md5($key);
		$randomids	= (array) $this->app->session->get('bookzo.discountorder.'.$key, array());
		$compare 	= array_diff($ids, $randomids);
		
		if ( !count($randomids) || count($compare) ) {
			$randomids = $ids;
			shuffle($randomids);
			$this->app->session->set('bookzo.discountorder.'.$key, $randomids);
		} 
		
		$return 	= array();
		foreach ( $randomids as $id ) {
			if ( array_key_exists($id, $items) ) {
				$return[$id] = $items[$id];
			}
		}
		
		return $return;
	}
}