<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

defined('_BOOKZO') or die;

/**
 * Bz data helper class.
 *
 * Helper for managing and retrieving paths
 * We can register paths.
 * When loading files we search paths top to bottom
 *
 * @subpackage 	framework.helpers
 * @since 		1.0.0
 */
class BzWidgetPriceList extends BWidget 
{
	public function render()
	{
		$params		= $this->params;
		$id 		= (int) $params->get('id');
		$columns 	= $params->get('type', 'columns') == 'columns';
		
		// get current search
		$active				= false;
		$searchstate 		= $this->app->registry->create( $this->app->session->get('bookzo.search.query') );
		$searchedarrival 	= $searchstate->get('arrival');

		// can we find this object
		if ( !$id || ($object = $this->app->api->getObjects($id)) === false ) {
			return '';
		}

		// get the prices for this object and active search
		$prices  		= $object->getPeriods($searchstate->get('persons',$object->get('persons', 2)), $searchstate->get('pets',0), $params->get('aid'));
		
		if ( !count($prices) ) {
			return '';
		}
		
		
		// build needed stuff
		$cols	  	= array();
		$dates	    = array();
		$mindate  	= false;
		$maxdate  	= false;
		
		foreach ( $prices as $i => $price ) 
		{
			$price 		= $this->app->registry->create($price);
			$nights 	= $price->get('nights');
			$special 	= $this->app->config->get('template.durationchange', 1) && ($price->get('week') || $price->get('midweek') || $price->get('weekend'));
			
			// check for validness
			if ( !$price->get('objectid') ) 
			{
				unset($prices[$i]);
				continue;
			}
			
			// do we want to see this period
			if ( $columns && !$special ) {
				if ( ($nights < 7 && $nights > $params->get('maxdays', 2)) ||
				     ($nights > 7 && ($nights%7 || ($nights/7 > $params->get('maxweeks', 1))))) {
					unset($prices[$i]);
					continue;
				}
			}
			
			$pricename 	= $price->get('specialname', $price->get('periodname'));			
			$month		= $this->app->date->create($price->get('arrival'))->format('Y-m-01');
			$arrival	= $price->get('arrival');
			
			if ( $columns ) {
				// create columns
				if ( !isset($cols[$pricename]) ) {
					$cols[$pricename] 				= new stdClass();
					$cols[$pricename]->nights	 	= $price->get('nights');
					$cols[$pricename]->arrivals 	= array();
				}
				
				$cols[$pricename]->arrivals[$arrival] = $price;

				// create months
				if ( !isset($dates[$month]) || !in_array($arrival, $dates[$month]) ) {
					$dates[$month][] = $arrival;
				}
			}
			else {
				$dates[$month][] = $this->app->registry->create($price);
			}
		}

		// sort cols by nights and let price link if columns
		if ( $columns ) {
			uasort($cols, function($a, $b) { return $a->nights > $b->nights; });
			$params->set('linkprice', true);
		}	
		
		
		return $this->renderer->render( ($columns ? 'default' : 'list' ), compact('dates','cols','active','params'));
	}
}
